unit hiGL_GMS;

interface

uses KOL,Windows,Share,Debug,OpenGL;

Type
    PGLVertex = ^TGLVertex;
    TGLVertex = record
       x,y,z : GLFloat;
    end;

    PGLVector = ^TGLVector;
    TGLVector = array[0..2] of GLFloat;

    PGLFace = ^TGLFace;
    TGLFace = array[0..2] of GLInt;

    PGLVertexArray = ^TGLVertexArray;
    TGLVertexArray = array[Word] of TGLVertex;

    PGLFacesArray = ^TGLFacesArray;
    TGLFacesArray = array[word] of TGLFace;

    THIGL_Mesh = class(TDebug)

      Vertices : PGLVertexArray; //  
      Faces : PGLFacesArray;   //  
      FasetNormals : PGLVertexArray; //   
      SmoothNormals : PGLVertexArray; //   
      VertexCount : Integer;  //  
      FacesCount : Integer;   //  
      fExtent : GLFloat;      //  
      Extent : GLBoolean;     //  
      public
       _prop_FileName  : string;
       _prop_Smooth  : Boolean;
       _prop_Extent    : Boolean;
       _data_FileName    : THI_Event;
       _event_onCreate   : THI_Event;
       _event_onRender   : THI_Event;
      
       procedure CalcNormals;
       procedure Draw;
       destructor Destroy; override;
    end;

    THIGL_GMS = class(TDebug)
 
      Vertices : PGLVertexArray; //  
      Meshes : TList;            // 
      CurrentFrame : Integer;    //  
      Action : Boolean;          //   (/)
      fExtent : GLFloat;         // 
      Extent : Boolean;          // ?
      fSmooth : Boolean;         // ?
      public
      _prop_FileName  : string;
      _prop_Smooth  : Boolean;
      _prop_Extent    : Boolean;
      _data_FileName    : THI_Event;
      _event_onCreate   : THI_Event;
      _event_onRender   : THI_Event;
      
      procedure _work_doLoad(var _Data:TData; Index:word);
      procedure _work_doRender(var _Data:TData; Index:word);
      constructor Create;
      destructor Destroy; override;
      published
    end;

implementation

procedure THIGL_Mesh.CalcNormals;
var
  i : Integer;
  wrki, vx1, vy1, vz1, vx2, vy2, vz2 : GLfloat;
  nx, ny, nz : GLfloat;
  wrkVector : TGLVertex;
  wrkVector1, wrkVector2, wrkVector3 : TGLVertex;
  wrkFace : TGLFace;
begin
  For i := 0 to FacesCount - 1 do begin
     wrkFace := Faces[i];
     wrkVector1 := Vertices[wrkFace[0]];
     wrkVector2 := Vertices[wrkFace[1]];
     wrkVector3 := Vertices[wrkFace[2]];

     vx1 := wrkVector1.x - wrkVector2.x;
     vy1 := wrkVector1.y - wrkVector2.y;
     vz1 := wrkVector1.z*fExtent - wrkVector2.z*fExtent;

     vx2 := wrkVector2.x - wrkVector3.x;
     vy2 := wrkVector2.y - wrkVector3.y;
     vz2 := wrkVector2.z - wrkVector3.z;

     //    
     nx := vy1 * vz2 - vz1 * vy2;
     ny := vz1 * vx2 - vx1 * vz2;
     nz := vx1 * vy2 - vy1 * vx2;

     //     
     wrki := sqrt (nx * nx + ny * ny + nz * nz);
     If wrki = 0 then wrki := 1; //     

     wrkVector.x := nx / wrki;
     wrkVector.y := ny / wrki;
     wrkVector.z := nz / wrki;

     FasetNormals[i] := wrkVector;
  end;
end;

procedure THIGL_Mesh.Draw;
var
   i : Integer;
   Face : TGLFace;
begin
  for i := 0 to FacesCount - 1 do begin
    glBegin(GL_TRIANGLES);
      Face := Faces[i];
      if _prop_Smooth then begin
        glNormal3fv(@SmoothNormals[Face[0]]);
        glVertex3fv(@Vertices[Face[0]]);
        glNormal3fv(@SmoothNormals[Face[1]]);
        glVertex3fv(@Vertices[Face[1]]);
        glNormal3fv(@SmoothNormals[Face[2]]);
        glVertex3fv(@Vertices[Face[2]]);
      end else begin
        glNormal3fv(@FasetNormals[i]);
        glVertex3fv(@Vertices[Face[0]]);
        glVertex3fv(@Vertices[Face[1]]);
        glVertex3fv(@Vertices[Face[2]]);
      end;
    glEnd;
  end;
end;

destructor THIGL_Mesh.Destroy;
begin
   FreeMem(Vertices,VertexCount*SizeOf(TGLVertex));
   FreeMem(SmoothNormals,VertexCount*SizeOf(TGLVertex));
   FreeMem(Faces,FacesCount*SizeOf(TGLFace));
   FreeMem(FasetNormals,FacesCount*SizeOf(TGLVector));
end;

procedure THIGL_GMS._work_doRender;
begin
  if Extent then
     begin
       fExtent := THIGL_Mesh(Meshes.Items[CurrentFrame]).fExtent;
       glScalef(fExtent,fExtent,fExtent);
     end;

  THIGL_Mesh(Meshes.Items[CurrentFrame]).Draw;

  if Action then begin
    inc(CurrentFrame);
    if CurrentFrame > (Meshes.Count - 1) then CurrentFrame := 0;
  end;
end;

procedure THIGL_GMS._work_doLoad;
var
   f : TextFile;
   S : String;
   filename : string;
   i : Integer;
   Vertex : TGLVertex;
   Face : TGLFace;
   Normal : TGLVertex;
   MaxVertex : GLFloat;
   NextMesh : THIGL_Mesh;
begin
   Meshes := TList.Create;
   filename := ReadFileName( ReadString(_data,_data_FileName,_prop_FileName));
   AssignFile(f,filename);
   Reset(f);
   While not Eof(f) do begin
     Readln(f,S);
     if S = 'New object' then 
     begin
        NextMesh := THIGL_Mesh.Create;
        repeat
          ReadLn(f, S);
        until (S = 'numverts numfaces') or eof(f);
        //     
        Readln(f,NextMesh.VertexCount,NextMesh.FacesCount);
        //     
        GetMem(NextMesh.Vertices,NextMesh.VertexCount*SizeOf(TGLVertex));
        GetMem(NextMesh.SmoothNormals,NextMesh.VertexCount*SizeOf(TGLVector));
        GetMem(NextMesh.Faces,NextMesh.FacesCount*SizeOf(TGLFace));
        GetMem(NextMesh.FasetNormals,NextMesh.FacesCount*SizeOf(TGLVector));

        ReadLn(f,S); //   Mesh vertices:
        //  
        for i := 0 to NextMesh.VertexCount - 1 do
          begin
            Readln(f,Vertex.x,Vertex.y,Vertex.z);
            NextMesh.Vertices[i] := Vertex;
          end;

        ReadLn(f,S); //   end vertices
        ReadLn(f,S); //   Mesh faces:
        //  
        for i := 0 to NextMesh.FacesCount - 1 do
          begin
            Readln(f,Face[0],Face[1],Face[2]);
            Face[0] := Face[0] - 1;
            Face[1] := Face[1] - 1;
            Face[2] := Face[2] - 1;
            NextMesh.Faces[i] := Face;
          end;
          ReadLn(f,S); //   end faces
          ReadLn(f,S); //   Faset Normals:
        for i := 0 to NextMesh.FacesCount - 1 do
          begin
            Readln(f,Normal.x,Normal.y,Normal.z);
            NextMesh.FasetNormals[i] := Normal;
          end;

          ReadLn(f,S); //   end faset normals
          ReadLn(f,S); //   SmoothNormals:
          //   
          for i := 0 to NextMesh.VertexCount - 1 do
            begin
              Readln(f,Normal.x,Normal.y,Normal.z);
              NextMesh.SmoothNormals[i] := Normal;
            end;
       //  
       MaxVertex := 0;
       for i := 0 to NextMesh.VertexCount - 1 do
         begin
         if MaxVertex > Vertices[i].x then MaxVertex := Vertices[i].x; 
         if MaxVertex > Vertices[i].y then MaxVertex := Vertices[i].y; 
         if MaxVertex > Vertices[i].z then MaxVertex := Vertices[i].z; 
         end;
       NextMesh.fExtent := 1/MaxVertex;

       //NextMesh.CalcNormals;

       Meshes.Add(NextMesh);
     end;
   end;
   CloseFile(f);
end;

constructor THIGL_GMS.Create;
begin
  Action := False;
  CurrentFrame := 0;
end;

destructor THIGL_GMS.Destroy;
Var i : Integer;
begin
  for i := 0 to Meshes.Count - 1 do
    begin
      THIGL_Mesh(Meshes.Items[i]).Destroy;
    end;
  Meshes.Free;
end;

end.